function [Data,CV1,CV2] = CMV2(x,y,ax,CV1_fold,CV2_fold,lv,varsel)

%   PLS-DA and cross-model validation (CMV); 
%   Model optimization (=CV1) and test sample prediction (=CV2);
%    
%
%   inputs:
%       x           ((I x 2) x J)     =   data
%       y           ((I x 2) x 1)     =   columnvector containing -1 and +1 defining which samples belong to which intervention
%       ax                (J x 1)     =   rowvector containing values x-axis
%       CV1_fold                      =   n_fold cross-validation in CV1
%       CV1_fold                      =   n_fold cross-validation in CV2
%       lv                            =   maximum number of latent variables
%       varsel            (1 x 5)     =   fraction of variables used in succesive variable selection rounds
%                                         e.g. [1;2;4;8;16] correspond with 100%,50%,25%,12.5% and 6.25% of all variables applied in the CMV
%                                                                                                                  
%
%   outputs:
%       Data structure
%       Data.x                    ((I x 2) x J) =   data
%       Data.y                    ((I x 2) x 1) =   columnvector containing -1 and +1 defining which samples belong to which intervention
%       Data.xaxis                      (1 x J) =   rowvector containing values x-axis
%
%       CV1 structure
%       CV1.ypred                 ((I x 2) x 1) =   columnvector containg predicted class labels in CV1
%       CV1.NMC                                 =   number of misclassifications in CV1
%       CV1.Q2                                  =   Q2 in CV1
%       CV1.AUROC                               =   area under the ROC curve in CV1
%
%       CV2 structure
%       CV2.ypred                 ((I x 2) x 1) =   columnvector containg predicted class labels in CV2
%       CV2.NMC                                 =   number of misclassifications in CV2
%       CV2.Q2                                  =   Q2 in CV2
%       CV2.AUROC                               =   area under the ROC curve in CV2
%       CV2.RP                          (1 x J) =   rowvector containing the rank product
%
%   
%       I/O: [Data,CV1,CV2] = CMV1(x,y,ax,CV1_fold,CV2_fold,lv,varsel)
%       Example [Data,CV1,CV2] = CMV1(x,y,ax,5,4,6,[1;2;4;8;16])
%
% ============================================================================
%
% This is a license to use and modify SOFTWARE  & DATA produced by:
% THE BIOSYSTEMS DATA ANALYSIS GROUP OF THE UNIVERSITEIT VAN AMSTERDAM
% 
% If you use, modify or redistribute the SOFTWARE & DATA and/or your source modifications, you agree: 
% 
% i)	to use the SOFTWARE & DATA and/or your source modifications solely as part of your research and not in any commercial product; 
% 	
% ii)	that the SOFTWARE  & DATA and/or your source modifications will not be distributed for profit; 
% 
% iii)	all copyright notices and this license note with the SOFTWARE & DATA are retained any  redistribution of the SOFTWARE & DATA, or any portion thereof; 
% 
% (iv)	to indemnify, hold harmless, and defend the "Biosystems Data Analysis Group of the Universiteit van Amsterdam" from and against any claims or lawsuits that arise or result from the use of the SOFTWARE & DATA or your source modifications. 
% 
% (v)	to properly reference the SOFTWARE & DATA when used in your reseach in any publication that may result from that research.
% 	
% Reserved Rights. The "Biosystems Data Analysis Group of the Universiteit van Amsterdam" retain title and all ownership rights to the SOFTWARE & DATA.
%
% ============================================================================


    % Init (1)
    [varsel,varblocks,xblock,xrows,xcolumns,variables]=CMV_init1(x,lv,varsel,CV2_fold);
    
    % Construction of designmatrix for 2CV   
    [CV2_Designmatrix,CV2_Designmatrix_rows]=CMV_designmatrix((xrows/2),CV2_fold);    
    
    % Double cross-validation
    for CV2loop=1:CV2_fold                        

        % Init (2)      
        [variables,varnr]=CMV_init2(xcolumns,variables);
        
        % CV1 split 1: restset and testset
        [variables,restset,testset]=CMV_CV1_split1(xblock,y,variables,CV2_Designmatrix,CV2loop,CV2_Designmatrix_rows);
                
        % Construction of designmatrix for 1CV 
        [CV1_Designmatrix]=CMV_designmatrix(restset.rows,CV1_fold);
                
        % Single cross-validation
        for CV1loop=1:CV1_fold                    
               
            % CV1 split 2: validationset and trainingset
            [validationset,trainingset] = CMV_CV1_split2(xblock,restset,y,CV1_Designmatrix,CV1loop);

            % Variation splitting (CV1)
            [trainingset,validationset]=CMV_CV1_MSCA(trainingset,validationset,CV1_Designmatrix,CV2_Designmatrix,CV1loop,CV2loop);
            
            % Autoscaling    
            [trainingset.Xwscaled,validationset.Xwscaled] = CMV_autoscaling(trainingset.Xw,validationset.Xw);
            
            % Variable selection
            for CV1_xblock=1:5                     
                                     
                % Init (3)
                [CV1_rank,nr_of_var] = CMV_init3(varblocks,lv,CV1_xblock);
                
                % Remove variables (backward elimination)
                [trainingset.Xwscaled,validationset.Xwscaled] = CMV_backward_elimination(nr_of_var,trainingset.Xwscaled,validationset.Xwscaled);
             
                % Calculate PLS regression coefficients b (1CV)
                [b] = pls(trainingset.Xwscaled,trainingset.y,lv) ;  
           
                % Predict y of validationset samples (all b's)
                variables.CV1.ypred.all(validationset.samples,1:lv,CV1_xblock) = validationset.Xwscaled * b';
                
                % Variable ranking of training and validation set according b
                [trainingset.Xwscaled,validationset.Xwscaled]=CMV_ranking_b(b,lv,nr_of_var,CV1_rank,trainingset.Xwscaled,validationset.Xwscaled);           
                    
            end

        end

        % Collect predicted y's from restset samples (CV1)
        [restset,variables]=CMV_CV1_ypredict(restset,y,lv,variables);
        
        % Find optimized CV1 parameters
        [variables]=CMV_CV1_optimization(variables);
               
        % Variation splitting
        [restset,testset]=CMV_CV2_MSCA(restset,testset,CV2_Designmatrix,CV2loop);
        
        % Autoscaling (CV2)
        [restset.Xwscaled,testset.Xwscaled] = CMV_autoscaling(restset.Xw,testset.Xw);

        % Collect predicted y's from testset samples
        [restset,testset,variables]=CMV_CV2_ypredict2(CV2loop,varblocks,varnr,restset,testset,variables);    

    end
     
    % CMV results  
    [variables]=CMV_results(y,variables);
    
    % CMV output 
    [Data,CV1,CV2]=CMV_output(x,y,ax,variables);

end